﻿using GalaSoft.MvvmLight;
using System.DirectoryServices;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.DirectoryServices.ActiveDirectory;

namespace AD_Explorer.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class DirectoryEntryViewModel : ViewModelBase
    {
        #region Attributes
        DirectoryEntry _entry;
        bool dummy;
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new DirectoryEntryViewModel
        /// </summary>
        /// <param name="entry">A DirectoryEntry that is encapsulated by this DirectoryEntryViewModel</param>
        public DirectoryEntryViewModel(DirectoryEntry entry)
        {
            this._entry = entry;
            IsExpanded = false;
        }
        
        /// <summary>
        /// Initializes a dummy DirectoryEntryViewModel so that Children are loaded JIT.
        /// </summary>
        private DirectoryEntryViewModel()
        {
            dummy = true;
        }
        #endregion
        
        #region Properties
        #region Children
        private ObservableCollection<DirectoryEntryViewModel> _children = new ObservableCollection<DirectoryEntryViewModel>();
        /// <summary>
        /// The <see cref="Children" /> property's name.
        /// </summary>
        public const string ChildrenPropertyName = "Children";
        public ObservableCollection<DirectoryEntryViewModel> Children
        {
            get
            {
                return _children;
            }
        }

        
        #endregion

        #region Name
        /// <summary>
        /// The <see cref="Name" /> property's name.
        /// </summary>
        public const string NamePropertyName = "Name";

        /// <summary>
        /// Gets the Name property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string Name
        {
            get
            {
                if (dummy) return "*";
                return (string)_entry.InvokeGet("CN");
            }
        }
        #endregion

        #region IsSelected
        /// <summary>
        /// The <see cref="IsSelected" /> property's name.
        /// </summary>
        public const string IsSelectedPropertyName = "IsSelected";

        private bool _isSelected = false;

        /// <summary>
        /// Gets the IsSelected property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsSelected
        {
            get
            {
                return _isSelected;
            }

            set
            {
                if (_isSelected == value)
                {
                    return;
                }
                _isSelected = value;
                // Update bindings, no broadcast
                RaisePropertyChanged(IsSelectedPropertyName);
            }
        }
        #endregion

        #region IsExpanded
        /// <summary>
        /// The <see cref="IsExpanded" /> property's name.
        /// </summary>
        public const string IsExpandedPropertyName = "IsExpanded";

        private bool _isExpanded = false;

        /// <summary>
        /// Gets the IsExpanded property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsExpanded
        {
            get
            {
                return _isExpanded;
            }

            set
            {
                _isExpanded = value;
                if (_isExpanded)
                {
                    DirectorySearcher searcher = new DirectorySearcher(_entry, "(|(objectClass=user)(objectClass=computer))");
                    searcher.SizeLimit = 50;
                    SearchResultCollection res = searcher.FindAll();
                    Children.Clear();
                    foreach (DirectoryEntry child in _entry.Children)
                    {
                        Children.Add(new DirectoryEntryViewModel(child));
                    }
                }
                else
                {
                    Children.Clear();
                    Children.Add(new DirectoryEntryViewModel());
                }
                // Update bindings, no broadcast
                RaisePropertyChanged(IsExpandedPropertyName);
            }
        }
        #endregion
        #endregion

        #region Methods

        #endregion

        public override void Cleanup()
        {
            base.Cleanup();
        }
    }
}